@@ -20,7 +20,7 @@
 #include <map>
 #include <set>
 #include <vector>
-#include <tr1/memory>
+#include "include/memory.h"
 #include <exception>
 
 #include "osd/osd_types.h"
@@ -99,7 +99,7 @@ class LFNIndex : public CollectionIndex {
   /// Path to Index base.
   const string base_path;
   /// For reference counting the collection @see Path
-  std::tr1::weak_ptr<CollectionIndex> self_ref;
+  ceph::weak_ptr<CollectionIndex> self_ref;
 
 protected:
   const uint32_t index_version;
@@ -155,7 +155,7 @@ public:
   virtual ~LFNIndex() {}
 
   /// @see CollectionIndex
-  void set_ref(std::tr1::shared_ptr<CollectionIndex> ref);
+  void set_ref(ceph::shared_ptr<CollectionIndex> ref);
 
   /// @see CollectionIndex
   int init();
@@ -165,48 +165,48 @@ public:
 
   /// @see CollectionIndex
   int created(
-    const hobject_t &hoid,
+    const ghobject_t &oid,
     const char *path
     );
 
   /// @see CollectionIndex
   int unlink(
-    const hobject_t &hoid
+    const ghobject_t &oid
     );
 
   /// @see CollectionIndex
   int lookup(
-    const hobject_t &hoid,
+    const ghobject_t &oid,
     IndexedPath *path,
     int *exist
     );
 
   /// @see CollectionIndex
   int collection_list(
-    vector<hobject_t> *ls
+    vector<ghobject_t> *ls
     );
 
   /// @see CollectionIndex
   int collection_list_partial(
-    const hobject_t &start,
+    const ghobject_t &start,
     int min_count,
     int max_count,
     snapid_t seq,
-    vector<hobject_t> *ls,
-    hobject_t *next
+    vector<ghobject_t> *ls,
+    ghobject_t *next
     );
 
   virtual int _split(
     uint32_t match,                             //< [in] value to match
     uint32_t bits,                              //< [in] bits to check
-    std::tr1::shared_ptr<CollectionIndex> dest  //< [in] destination index
+    ceph::shared_ptr<CollectionIndex> dest  //< [in] destination index
     ) = 0;
   
   /// @see CollectionIndex
   int split(
     uint32_t match,
     uint32_t bits,
-    std::tr1::shared_ptr<CollectionIndex> dest
+    ceph::shared_ptr<CollectionIndex> dest
     ) {
     WRAP_RETRY(
       r = _split(match, bits, dest);
@@ -221,20 +221,20 @@ protected:
   /// Will be called upon object creation
   virtual int _created(
     const vector<string> &path, ///< [in] Path to subdir.
-    const hobject_t &hoid,      ///< [in] Object created.
+    const ghobject_t &oid,      ///< [in] Object created.
     const string &mangled_name  ///< [in] Mangled filename.
     ) = 0;
 
   /// Will be called to remove an object
   virtual int _remove(
     const vector<string> &path,     ///< [in] Path to subdir.
-    const hobject_t &hoid,          ///< [in] Object to remove.
+    const ghobject_t &oid,          ///< [in] Object to remove.
     const string &mangled_name	    ///< [in] Mangled filename.
     ) = 0;
 
-  /// Return the path and mangled_name for hoid.
+  /// Return the path and mangled_name for oid.
   virtual int _lookup(
-    const hobject_t &hoid,///< [in] Object for lookup.
+    const ghobject_t &oid,///< [in] Object for lookup.
     vector<string> *path, ///< [out] Path to the object.
     string *mangled_name, ///< [out] Mangled filename.
     int *exists		  ///< [out] True if the object exists.
@@ -252,17 +252,17 @@ protected:
    */
   /// List contents of collection.
   virtual int _collection_list(
-    vector<hobject_t> *ls ///< [out] Listed objects.
+    vector<ghobject_t> *ls ///< [out] Listed objects.
     ) = 0;
 
   /// @see CollectionIndex
   virtual int _collection_list_partial(
-    const hobject_t &start,
+    const ghobject_t &start,
     int min_count,
     int max_count,
     snapid_t seq,
-    vector<hobject_t> *ls,
-    hobject_t *next
+    vector<ghobject_t> *ls,
+    ghobject_t *next
     ) = 0;
 
 protected:
@@ -278,8 +278,8 @@ protected:
   int link_object(
     const vector<string> &from,   ///< [in] Source subdirectory.
     const vector<string> &to,     ///< [in] Dest subdirectory.
-    const hobject_t &hoid,        ///< [in] Object to move.
-    const string &from_short_name ///< [in] Mangled filename of hoid.
+    const ghobject_t &oid,        ///< [in] Object to move.
+    const string &from_short_name ///< [in] Mangled filename of oid.
     ); ///< @return Error Code, 0 on success
 
   /**
@@ -296,8 +296,8 @@ protected:
    */
   int remove_objects(
     const vector<string> &dir,             
-    const map<string, hobject_t> &to_remove,
-    map<string, hobject_t> *remaining
+    const map<string, ghobject_t> &to_remove,
+    map<string, ghobject_t> *remaining
     );
 	
 
@@ -322,11 +322,11 @@ protected:
    */
   int remove_object(
     const vector<string> &from,  ///< [in] Directory from which to remove.
-    const hobject_t &to_remove   ///< [in] Object to remove.
+    const ghobject_t &to_remove   ///< [in] Object to remove.
     );
 
   /**
-   * Gets the filename corresponding to hoid in from.
+   * Gets the filename corresponding to oid in from.
    * 
    * The filename may differ between subdirectories.  Furthermore,
    * file creations ore removals in from may invalidate the name.
@@ -334,7 +334,7 @@ protected:
    */
   int get_mangled_name(
     const vector<string> &from, ///< [in] Subdirectory
-    const hobject_t &hoid,	///< [in] Object 
+    const ghobject_t &oid,	///< [in] Object
     string *mangled_name,	///< [out] Filename
     int *exists			///< [out] 1 if the file exists, else 0
     );
@@ -352,7 +352,7 @@ protected:
     LFNIndex &from,             ///< [in] from index
     LFNIndex &dest,             ///< [in] to index
     const vector<string> &path, ///< [in] path to split
-    const pair<string, hobject_t> &obj ///< [in] obj to move
+    const pair<string, ghobject_t> &obj ///< [in] obj to move
     );
 
   /**
@@ -369,7 +369,7 @@ protected:
     const vector<string> &to_list,
     int max_objects,
     long *handle,
-    map<string, hobject_t> *out
+    map<string, ghobject_t> *out
     );
 
   /// Lists subdirectories.
@@ -425,43 +425,43 @@ private:
   }
 
   /**
-   * Gets the filename corresponsing to hoid in path.
+   * Gets the filename corresponsing to oid in path.
    *
-   * @param [in] path Path in which to get filename for hoid.
-   * @param [in] hoid Object for which to get filename.
-   * @param [out] mangled_name Filename for hoid, pass NULL if not needed.
-   * @param [out] full_path Fullpath for hoid, pass NULL if not needed.
+   * @param [in] path Path in which to get filename for oid.
+   * @param [in] oid Object for which to get filename.
+   * @param [out] mangled_name Filename for oid, pass NULL if not needed.
+   * @param [out] full_path Fullpath for oid, pass NULL if not needed.
    * @param [out] exists 1 if the file exists, 0 otherwise, pass NULL if 
    * not needed
    * @return Error Code, 0 on success.
    */
   int lfn_get_name(
     const vector<string> &path,
-    const hobject_t &hoid, 
+    const ghobject_t &oid,
     string *mangled_name,
     string *full_path,
     int *exists
     );
 
-  /// Adjusts path contents when hoid is created at name mangled_name.
+  /// Adjusts path contents when oid is created at name mangled_name.
   int lfn_created(
     const vector<string> &path, ///< [in] Path to adjust.
-    const hobject_t &hoid,	///< [in] Object created. 
+    const ghobject_t &oid,	///< [in] Object created.
     const string &mangled_name  ///< [in] Filename of created object.
     );
 
-  /// Removes hoid from path while adjusting path contents
+  /// Removes oid from path while adjusting path contents
   int lfn_unlink(
-    const vector<string> &path, ///< [in] Path containing hoid.
-    const hobject_t &hoid,	///< [in] Object to remove.
+    const vector<string> &path, ///< [in] Path containing oid.
+    const ghobject_t &oid,	///< [in] Object to remove.
     const string &mangled_name	///< [in] Filename of object to remove.
     );
 
-  ///Transate a file into and hobject_t.
+  ///Transate a file into and ghobject_t.
   int lfn_translate(
     const vector<string> &path, ///< [in] Path containing the file.
     const string &short_name,	///< [in] Filename to translate. 
-    hobject_t *out		///< [out] Object found.
+    ghobject_t *out		///< [out] Object found.
     ); ///< @return Negative error code on error, 0 if not an object, 1 else
 
   /* manglers/demanglers */
@@ -478,35 +478,35 @@ private:
 
   /// Generate object name
   string lfn_generate_object_name_keyless(
-    const hobject_t &hoid ///< [in] Object for which to generate.
+    const ghobject_t &oid ///< [in] Object for which to generate.
     ); ///< @return Generated object name.
 
   /// Generate object name
   string lfn_generate_object_name_poolless(
-    const hobject_t &hoid ///< [in] Object for which to generate.
+    const ghobject_t &oid ///< [in] Object for which to generate.
     ); ///< @return Generated object name.
 
   /// Generate object name
   string lfn_generate_object_name(
-    const hobject_t &hoid ///< [in] Object for which to generate.
+    const ghobject_t &oid ///< [in] Object for which to generate.
     ); ///< @return Generated object name.
 
   /// Parse object name
   bool lfn_parse_object_name_keyless(
     const string &long_name, ///< [in] Name to parse
-    hobject_t *out	     ///< [out] Resulting Object
+    ghobject_t *out	     ///< [out] Resulting Object
     ); ///< @return True if successfull, False otherwise.
 
   /// Parse object name
   bool lfn_parse_object_name_poolless(
     const string &long_name, ///< [in] Name to parse
-    hobject_t *out	     ///< [out] Resulting Object
+    ghobject_t *out	     ///< [out] Resulting Object
     ); ///< @return True if successfull, False otherwise.
 
   /// Parse object name
   bool lfn_parse_object_name(
     const string &long_name, ///< [in] Name to parse
-    hobject_t *out	     ///< [out] Resulting Object
+    ghobject_t *out	     ///< [out] Resulting Object
     ); ///< @return True if successfull, False otherwise.
 
   /// Checks whether short_name is a hashed filename.
@@ -521,7 +521,7 @@ private:
 
   /// Generate hashed name.
   string lfn_get_short_name(
-    const hobject_t &hoid, ///< [in] Object for which to generate.
+    const ghobject_t &oid, ///< [in] Object for which to generate.
     int i		   ///< [in] Index of hashed name to generate.
     ); ///< @return Hashed filename.
 
@@ -554,7 +554,7 @@ private:
   int decompose_full_path(
     const char *in,      ///< [in] Full path to object.
     vector<string> *out, ///< [out] Path to object at in.
-    hobject_t *hoid,	 ///< [out] Object at in.
+    ghobject_t *oid,	 ///< [out] Object at in.
     string *shortname	 ///< [out] Filename of object at in.
     ); ///< @return Error Code, 0 on success.
 
